@@ -9,9 +9,9 @@
 //!
 //! ### Rust Version
 //!
-//! This version of fixedbitset requires Rust 1.31 or later.
+//! This version of fixedbitset requires Rust 1.39 or later.
 //!
-#![doc(html_root_url="https://docs.rs/fixedbitset/0.3.1/")]
+#![doc(html_root_url="https://docs.rs/fixedbitset/0.4.0/")]
 
 #![cfg_attr(not(feature = "std"), no_std)]
 
@@ -28,6 +28,11 @@ use core as std;
 
 mod range;
 
+#[cfg(feature = "serde")]
+extern crate serde;
+#[cfg(feature = "serde")]
+use serde::{Serialize, Deserialize};
+
 use std::fmt::Write;
 use std::fmt::{Display, Error, Formatter, Binary};
 
@@ -51,6 +56,7 @@ fn div_rem(x: usize, d: usize) -> (usize, usize)
 /// The bit set has a fixed capacity in terms of enabling bits (and the
 /// capacity can grow using the `grow` method).
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Default)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct FixedBitSet {
     data: Vec<Block>,
     /// length in bits
@@ -59,6 +65,15 @@ pub struct FixedBitSet {
 
 impl FixedBitSet
 {
+    /// Create a new empty **FixedBitSet**.
+    pub const fn new() -> Self
+    {
+        FixedBitSet {
+            data: Vec::new(),
+            length: 0,
+        }
+    }
+
     /// Create a new **FixedBitSet** with a specific number of bits,
     /// all initially clear.
     pub fn with_capacity(bits: usize) -> Self
@@ -70,13 +85,14 @@ impl FixedBitSet
             length: bits,
         }
     }
+
     /// Create a new **FixedBitSet** with a specific number of bits,
     /// initialized from provided blocks.
     ///
     /// If the blocks are not the exact size needed for the capacity
     /// they will be padded with zeros (if shorter) or truncated to
     /// the capacity (if longer).
-    /// 
+    ///
     /// For example:
     /// ```
     /// let data = vec![4];
@@ -302,17 +318,15 @@ impl FixedBitSet
         match self.as_slice().split_first() {
             Some((&block, rem)) => {
                 Ones {
-                    current_bit_idx: 0,
-                    current_block_idx: 0,
-                    current_block: block,
+                    bitset: block,
+                    block_idx: 0,
                     remaining_blocks: rem
                 }
             }
             None => {
                 Ones {
-                    current_bit_idx: 0,
-                    current_block_idx: 0,
-                    current_block: 0,
+                    bitset: 0,
+                    block_idx: 0,
                     remaining_blocks: &[]
                 }
             }
@@ -593,10 +607,9 @@ impl Iterator for Masks {
 ///
 /// This struct is created by the [`FixedBitSet::ones`] method.
 pub struct Ones<'a> {
-    current_bit_idx: usize,
-    current_block_idx: usize,
+    bitset: Block,
+    block_idx: usize,
     remaining_blocks: &'a [Block],
-    current_block: Block
 }
 
 impl<'a> Iterator for Ones<'a> {
@@ -604,38 +617,18 @@ impl<'a> Iterator for Ones<'a> {
 
     #[inline]
     fn next(&mut self) -> Option<Self::Item> {
-        let mut block = self.current_block;
-        let mut idx = self.current_bit_idx;
-
-        loop {
-            loop {
-                if (block & 1) == 1 {
-                    self.current_block = block >> 1;
-                    self.current_bit_idx = idx + 1;
-                    return Some(idx);
-                }
-                // reordering the two lines below makes a huge (2x) difference in performance!
-                block = block >> 1;
-                idx += 1;
-                if block == 0 {
-                    break;
-                }
-            }
-
-            // go to next block
-            match self.remaining_blocks.split_first() {
-                Some((&next_block, rest)) => {
-                    self.remaining_blocks = rest;
-                    self.current_block_idx += 1;
-                    idx = self.current_block_idx * BITS;
-                    block = next_block;
-                }
-                None => {
-                    // last block => done
-                    return None;
-                }
+        while self.bitset == 0 {
+            if self.remaining_blocks.is_empty() {
+                return None;
             }
+            self.bitset = self.remaining_blocks[0];
+            self.remaining_blocks = &self.remaining_blocks[1..];
+            self.block_idx += 1;
         }
+        let t = self.bitset & (0 as Block).wrapping_sub(self.bitset);
+        let r = self.bitset.trailing_zeros() as usize;
+        self.bitset ^= t;
+        Some(self.block_idx * BITS + r)
     }
 }
 
@@ -1596,3 +1589,15 @@ fn display_trait() {
     assert_eq!(format!("{}", fb), "00101000");
     assert_eq!(format!("{:#}", fb), "0b00101000");
 }
+
+#[test]         
+#[cfg(feature="serde")]
+fn test_serialize() {                                               
+    let mut fb = FixedBitSet::with_capacity(10);
+    fb.put(2);                               
+    fb.put(3);
+    fb.put(6);                                      
+    fb.put(8);                       
+    let serialized = serde_json::to_string(&fb).unwrap();
+    assert_eq!(r#"{"data":[332],"length":10}"#, serialized);
+}
